@@ -11,19 +11,18 @@
 #' @return a tibble with 2 columns and as many rows as needed
 #' @noRd
 get_parameters_raw <- function(
-  parameter = "hs",
-  node = 42,
-  start = as.POSIXct("1994-01-01Z00:00:00"),
-  end = as.POSIXct("1994-12-31Z23:00:00")
-) {
+    parameter = "hs",
+    node = 42,
+    start = as.POSIXct("1994-01-01Z00:00:00", tz = "UTC"),
+    end = as.POSIXct("1994-12-31Z23:00:00", tz = "UTC")) {
   if (parameter == "tp") {
     single_parameter <- "fp"
   } else {
     single_parameter <- parameter
   }
 
-  start_str <- strftime(start, format = "%Y-%m-%dT%H:%M:%SZ")
-  end_str <- strftime(end, format = "%Y-%m-%dT%H:%M:%SZ")
+  start_str <- strftime(start, format = "%Y-%m-%dT%H:%M:%SZ", tz = "UTC")
+  end_str <- strftime(end, format = "%Y-%m-%dT%H:%M:%SZ", tz = "UTC")
 
   # Cassandra database start indexing at 1, so decrements node number
   node <- node - 1
@@ -77,25 +76,32 @@ get_parameters_raw <- function(
 #' @return a tibble with N-rows and `length(parameters)` columns.
 #' @export
 #'
-#' @examplesIf requireNamespace("resourcecodedata", quietly = TRUE)
+#' @examplesIf curl::has_internet()
 #' ts <- get_parameters(parameters = c("hs", "tp"), node = 42)
 #' plot(ts$time, ts$hs, type = "l")
 get_parameters <- function(
-  parameters = "hs",
-  node = 42,
-  start = as.POSIXct("1994-01-01 00:00:00", tz = "UTC"),
-  end = as.POSIXct("1994-12-31 23:00:00", tz = "UTC")
-) {
-  has_data()
-
+    parameters = "hs",
+    node = 42,
+    start = as.POSIXct("1994-01-01 00:00:00", tz = "UTC"),
+    end = as.POSIXct("1994-12-31 23:00:00", tz = "UTC")) {
   parameters <- tolower(parameters)
 
-  stopifnot(parameters %in% c("tp", resourcecodedata::rscd_variables$name))
+  if (any(parameters %nin% c("tp", resourcecodedata::rscd_variables$name))) {
+    errors <- parameters[parameters %nin% c("tp", resourcecodedata::rscd_variables$name)]
+    stop(
+      "Requested parameters do not exists in the database: ",
+      paste0(errors, collapse = ", "), "."
+    )
+  }
 
   node <- as.integer(node)
 
-  stopifnot(length(node) == 1)
-  stopifnot(node %in% resourcecodedata::rscd_field$node)
+  if (length(node) != 1) {
+    stop("The function can retreive only one location a time.")
+  }
+  if (node %nin% resourcecodedata::rscd_field$node) {
+    stop("The requested location do no exist in the database.")
+  }
 
   if (is.character(start)) {
     start <- as.POSIXct(start, tz = "UTC")
@@ -119,8 +125,31 @@ get_parameters <- function(
     )
   }
 
-  stopifnot(start >= rscd_casandra_start_date)
-  stopifnot(end <= rscd_casandra_end_date)
+  if (start < rscd_casandra_start_date) {
+    stop(
+      "'start' is outside the covered period: ",
+      paste(
+        format(c(rscd_casandra_start_date, rscd_casandra_end_date),
+          format = "%Y-%m-%d %H:%M %Z"
+        ),
+        collapse = " \u2014 "
+      )
+    )
+  }
+  if (end > rscd_casandra_end_date) {
+    stop(
+      "'end' is outside the covered period: ",
+      paste(
+        format(c(rscd_casandra_start_date, rscd_casandra_end_date),
+          format = "%Y-%m-%d %H:%M %Z"
+        ),
+        collapse = " \u2014 "
+      )
+    )
+  }
+  if (start >= end) {
+    stop("'end' must be after 'start'")
+  }
 
   out <- get_parameters_raw(
     parameters[1],
